@@ -1,13 +1,21 @@
 # -*- coding: utf-8 -*-
 
-# ########################## Copyrights and license ############################
+############################ Copyrights and license ############################
 #                                                                              #
 # Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
 # Copyright 2012 Zearin <zearin@gonk.net>                                      #
 # Copyright 2013 AKFish <akfish@gmail.com>                                     #
 # Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
+# Copyright 2014 Andrew Scheller <github@loowis.durge.org>                     #
+# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
+# Copyright 2016 Jakub Wilk <jwilk@jwilk.net>                                  #
+# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
+# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
+# Copyright 2016 Sam Corbett <sam.corbett@cloudsoftcorp.com>                   #
+# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
 #                                                                              #
-# This file is part of PyGithub. http://jacquev6.github.com/PyGithub/          #
+# This file is part of PyGithub.                                               #
+# http://pygithub.readthedocs.io/                                              #
 #                                                                              #
 # PyGithub is free software: you can redistribute it and/or modify it under    #
 # the terms of the GNU Lesser General Public License as published by the Free  #
@@ -22,12 +30,15 @@
 # You should have received a copy of the GNU Lesser General Public License     #
 # along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
 #                                                                              #
-# ##############################################################################
+################################################################################
 
+from __future__ import absolute_import
 import datetime
+from operator import itemgetter
 
-import GithubException
-import Consts
+from . import GithubException
+from . import Consts
+import six
 
 
 class _NotSetType:
@@ -35,6 +46,8 @@ class _NotSetType:
         return "NotSet"
 
     value = None
+
+
 NotSet = _NotSetType()
 
 
@@ -126,18 +139,18 @@ class GithubObject(object):
         elif isinstance(value, type):
             try:
                 return _ValuedAttribute(transform(value))
-            except Exception, e:
+            except Exception as e:
                 return _BadAttribute(value, type, e)
         else:
             return _BadAttribute(value, type)
 
     @staticmethod
     def _makeStringAttribute(value):
-        return GithubObject.__makeSimpleAttribute(value, (str, unicode))
+        return GithubObject.__makeSimpleAttribute(value, (str, six.text_type))
 
     @staticmethod
     def _makeIntAttribute(value):
-        return GithubObject.__makeSimpleAttribute(value, (int, long))
+        return GithubObject.__makeSimpleAttribute(value, six.integer_types)
 
     @staticmethod
     def _makeBoolAttribute(value):
@@ -149,7 +162,7 @@ class GithubObject(object):
 
     @staticmethod
     def _makeTimestampAttribute(value):
-        return GithubObject.__makeTransformedAttribute(value, (int, long), datetime.datetime.utcfromtimestamp)
+        return GithubObject.__makeTransformedAttribute(value, six.integer_types, datetime.datetime.utcfromtimestamp)
 
     @staticmethod
     def _makeDatetimeAttribute(value):
@@ -158,24 +171,28 @@ class GithubObject(object):
                 # The Downloads API has been removed. I'm keeping this branch because I have no mean
                 # to check if it's really useless now.
                 return datetime.datetime.strptime(s, "%Y-%m-%dT%H:%M:%S.000Z")  # pragma no cover (This branch was used only when creating a download)
-            elif len(s) == 25:
+            elif len(s) >= 25:
                 return datetime.datetime.strptime(s[:19], "%Y-%m-%dT%H:%M:%S") + (1 if s[19] == '-' else -1) * datetime.timedelta(hours=int(s[20:22]), minutes=int(s[23:25]))
             else:
                 return datetime.datetime.strptime(s, "%Y-%m-%dT%H:%M:%SZ")
 
-        return GithubObject.__makeTransformedAttribute(value, (str, unicode), parseDatetime)
+        return GithubObject.__makeTransformedAttribute(value, (str, six.text_type), parseDatetime)
 
     def _makeClassAttribute(self, klass, value):
         return GithubObject.__makeTransformedAttribute(value, dict, lambda value: klass(self._requester, self._headers, value, completed=False))
 
     @staticmethod
     def _makeListOfStringsAttribute(value):
-        return GithubObject.__makeSimpleListAttribute(value, (str, unicode))
+        return GithubObject.__makeSimpleListAttribute(value, (str, six.text_type))
 
     @staticmethod
     def _makeListOfIntsAttribute(value):
         return GithubObject.__makeSimpleListAttribute(value, int)
 
+    @staticmethod
+    def _makeListOfDictsAttribute(value):
+        return GithubObject.__makeSimpleListAttribute(value, dict)
+
     @staticmethod
     def _makeListOfListOfStringsAttribute(value):
         return GithubObject.__makeSimpleListAttribute(value, list)
@@ -187,24 +204,41 @@ class GithubObject(object):
             return _BadAttribute(value, [dict])
 
     def _makeDictOfStringsToClassesAttribute(self, klass, value):
-        if isinstance(value, dict) and all(isinstance(key, (str, unicode)) and isinstance(element, dict) for key, element in value.iteritems()):
-            return _ValuedAttribute(dict((key, klass(self._requester, self._headers, element, completed=False)) for key, element in value.iteritems()))
+        if isinstance(value, dict) and all(isinstance(key, (str, six.text_type)) and isinstance(element, dict) for key, element in six.iteritems(value)):
+            return _ValuedAttribute(dict((key, klass(self._requester, self._headers, element, completed=False)) for key, element in six.iteritems(value)))
         else:
-            return _BadAttribute(value, {(str, unicode): dict})
+            return _BadAttribute(value, {(str, six.text_type): dict})
 
     @property
     def etag(self):
         '''
-        :type str
+        :type: str
         '''
         return self._headers.get(Consts.RES_ETAG)
 
     @property
     def last_modified(self):
         '''
-        :type str
+        :type: str
         '''
-        return self._headers.get(Consts.RES_LAST_MODIFED)
+        return self._headers.get(Consts.RES_LAST_MODIFIED)
+
+    def get__repr__(self, params):
+        """
+        Converts the object to a nicely printable string.
+        """
+        def format_params(params):
+            items = list(params.items())
+            for k, v in sorted(items, key=itemgetter(0), reverse=True):
+                if isinstance(v, bytes):
+                    v = v.decode('utf-8')
+                if isinstance(v, six.text_type):
+                    v = u'"{v}"'.format(v=v)
+                yield u'{k}={v}'.format(k=k, v=v)
+        return u'{class_name}({params})'.format(
+            class_name=self.__class__.__name__,
+            params=u", ".join(list(format_params(params)))
+        )
 
 
 class NonCompletableGithubObject(GithubObject):
@@ -232,6 +266,8 @@ class CompletableGithubObject(GithubObject):
             self.__complete()
 
     def __complete(self):
+        if self._url.value is None:
+            raise GithubException.IncompletableObject(400, "Returned object contains no URL")
         headers, data = self._requester.requestJsonAndCheck(
             "GET",
             self._url.value
