@@ -7,8 +7,20 @@
 # Copyright 2012 Zearin <zearin@gonk.net>                                      #
 # Copyright 2013 AKFish <akfish@gmail.com>                                     #
 # Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
+# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
+# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
+# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
+# Copyright 2017 Simon <spam@esemi.ru>                                         #
+# Copyright 2018 Iraquitan Cordeiro Filho <iraquitanfilho@gmail.com>           #
+# Copyright 2018 Steve Kowalik <steven@wedontsleep.org>                        #
+# Copyright 2018 Victor Granic <vmg@boreal321.com>                             #
+# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
+# Copyright 2018 namc <namratachaudhary@users.noreply.github.com>              #
+# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
+# Copyright 2018 itsbruce <it.is.bruce@gmail.com>                              #
 #                                                                              #
-# This file is part of PyGithub. http://jacquev6.github.com/PyGithub/          #
+# This file is part of PyGithub.                                               #
+# http://pygithub.readthedocs.io/                                              #
 #                                                                              #
 # PyGithub is free software: you can redistribute it and/or modify it under    #
 # the terms of the GNU Lesser General Public License as published by the Free  #
@@ -25,29 +37,51 @@
 #                                                                              #
 ################################################################################
 
+from __future__ import absolute_import
+import datetime
+
 import github.GithubObject
 import github.PaginatedList
 
 import github.Gist
 import github.Repository
 import github.NamedUser
+import github.Permissions
 import github.Plan
 import github.Organization
 import github.Event
+import six
 
 
 class NamedUser(github.GithubObject.CompletableGithubObject):
     """
-    This class represents NamedUsers as returned for example by http://developer.github.com/v3/todo
+    This class represents NamedUsers. The reference can be found here https://developer.github.com/v3/users/#get-a-single-user
     """
 
+    def __repr__(self):
+        return self.get__repr__({"login": self._login.value})
+
+    @property
+    def node_id(self):
+        """
+        :type: string
+        """
+        self._completeIfNotSet(self._node_id)
+        return self._node_id.value
+
+    def __hash__(self):
+        return hash((self.id, self.login))
+
+    def __eq__(self, other):
+        return isinstance(other, type(self)) and self.login == other.login and self.id == other.id
+
     @property
     def avatar_url(self):
         """
         :type: string
         """
         self._completeIfNotSet(self._avatar_url)
-        return self._NoneIfNotSet(self._avatar_url)
+        return self._avatar_url.value
 
     @property
     def bio(self):
@@ -55,7 +89,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._bio)
-        return self._NoneIfNotSet(self._bio)
+        return self._bio.value
 
     @property
     def blog(self):
@@ -63,7 +97,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._blog)
-        return self._NoneIfNotSet(self._blog)
+        return self._blog.value
 
     @property
     def collaborators(self):
@@ -71,7 +105,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: integer
         """
         self._completeIfNotSet(self._collaborators)
-        return self._NoneIfNotSet(self._collaborators)
+        return self._collaborators.value
 
     @property
     def company(self):
@@ -79,7 +113,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._company)
-        return self._NoneIfNotSet(self._company)
+        return self._company.value
 
     @property
     def contributions(self):
@@ -87,7 +121,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: integer
         """
         self._completeIfNotSet(self._contributions)
-        return self._NoneIfNotSet(self._contributions)
+        return self._contributions.value
 
     @property
     def created_at(self):
@@ -95,7 +129,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: datetime.datetime
         """
         self._completeIfNotSet(self._created_at)
-        return self._NoneIfNotSet(self._created_at)
+        return self._created_at.value
 
     @property
     def disk_usage(self):
@@ -103,7 +137,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: integer
         """
         self._completeIfNotSet(self._disk_usage)
-        return self._NoneIfNotSet(self._disk_usage)
+        return self._disk_usage.value
 
     @property
     def email(self):
@@ -111,7 +145,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._email)
-        return self._NoneIfNotSet(self._email)
+        return self._email.value
 
     @property
     def events_url(self):
@@ -119,7 +153,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._events_url)
-        return self._NoneIfNotSet(self._events_url)
+        return self._events_url.value
 
     @property
     def followers(self):
@@ -127,7 +161,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: integer
         """
         self._completeIfNotSet(self._followers)
-        return self._NoneIfNotSet(self._followers)
+        return self._followers.value
 
     @property
     def followers_url(self):
@@ -135,7 +169,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._followers_url)
-        return self._NoneIfNotSet(self._followers_url)
+        return self._followers_url.value
 
     @property
     def following(self):
@@ -143,7 +177,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: integer
         """
         self._completeIfNotSet(self._following)
-        return self._NoneIfNotSet(self._following)
+        return self._following.value
 
     @property
     def following_url(self):
@@ -151,7 +185,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._following_url)
-        return self._NoneIfNotSet(self._following_url)
+        return self._following_url.value
 
     @property
     def gists_url(self):
@@ -159,7 +193,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._gists_url)
-        return self._NoneIfNotSet(self._gists_url)
+        return self._gists_url.value
 
     @property
     def gravatar_id(self):
@@ -167,7 +201,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._gravatar_id)
-        return self._NoneIfNotSet(self._gravatar_id)
+        return self._gravatar_id.value
 
     @property
     def hireable(self):
@@ -175,7 +209,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: bool
         """
         self._completeIfNotSet(self._hireable)
-        return self._NoneIfNotSet(self._hireable)
+        return self._hireable.value
 
     @property
     def html_url(self):
@@ -183,7 +217,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._html_url)
-        return self._NoneIfNotSet(self._html_url)
+        return self._html_url.value
 
     @property
     def id(self):
@@ -191,7 +225,23 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: integer
         """
         self._completeIfNotSet(self._id)
-        return self._NoneIfNotSet(self._id)
+        return self._id.value
+
+    @property
+    def invitation_teams_url(self):
+        """
+        :type: string
+        """
+        self._completeIfNotSet(self._invitation_teams_url)
+        return self._invitation_teams_url.value
+
+    @property
+    def inviter(self):
+        """
+        :type: github.NamedUser.NamedUser
+        """
+        self._completeIfNotSet(self._inviter)
+        return self._inviter.value
 
     @property
     def location(self):
@@ -199,7 +249,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._location)
-        return self._NoneIfNotSet(self._location)
+        return self._location.value
 
     @property
     def login(self):
@@ -207,7 +257,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._login)
-        return self._NoneIfNotSet(self._login)
+        return self._login.value
 
     @property
     def name(self):
@@ -215,7 +265,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._name)
-        return self._NoneIfNotSet(self._name)
+        return self._name.value
 
     @property
     def organizations_url(self):
@@ -223,7 +273,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._organizations_url)
-        return self._NoneIfNotSet(self._organizations_url)
+        return self._organizations_url.value
 
     @property
     def owned_private_repos(self):
@@ -231,7 +281,15 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: integer
         """
         self._completeIfNotSet(self._owned_private_repos)
-        return self._NoneIfNotSet(self._owned_private_repos)
+        return self._owned_private_repos.value
+
+    @property
+    def permissions(self):
+        """
+        :type: :class:`github.Permissions.Permissions`
+        """
+        self._completeIfNotSet(self._permissions)
+        return self._permissions.value
 
     @property
     def plan(self):
@@ -239,7 +297,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: :class:`github.Plan.Plan`
         """
         self._completeIfNotSet(self._plan)
-        return self._NoneIfNotSet(self._plan)
+        return self._plan.value
 
     @property
     def private_gists(self):
@@ -247,7 +305,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: integer
         """
         self._completeIfNotSet(self._private_gists)
-        return self._NoneIfNotSet(self._private_gists)
+        return self._private_gists.value
 
     @property
     def public_gists(self):
@@ -255,7 +313,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: integer
         """
         self._completeIfNotSet(self._public_gists)
-        return self._NoneIfNotSet(self._public_gists)
+        return self._public_gists.value
 
     @property
     def public_repos(self):
@@ -263,7 +321,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: integer
         """
         self._completeIfNotSet(self._public_repos)
-        return self._NoneIfNotSet(self._public_repos)
+        return self._public_repos.value
 
     @property
     def received_events_url(self):
@@ -271,7 +329,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._received_events_url)
-        return self._NoneIfNotSet(self._received_events_url)
+        return self._received_events_url.value
 
     @property
     def repos_url(self):
@@ -279,7 +337,23 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._repos_url)
-        return self._NoneIfNotSet(self._repos_url)
+        return self._repos_url.value
+
+    @property
+    def role(self):
+        """
+        :type: string
+        """
+        self._completeIfNotSet(self._role)
+        return self._role.value
+
+    @property
+    def site_admin(self):
+        """
+        :type: bool
+        """
+        self._completeIfNotSet(self._site_admin)
+        return self._site_admin.value
 
     @property
     def starred_url(self):
@@ -287,7 +361,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._starred_url)
-        return self._NoneIfNotSet(self._starred_url)
+        return self._starred_url.value
 
     @property
     def subscriptions_url(self):
@@ -295,7 +369,23 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._subscriptions_url)
-        return self._NoneIfNotSet(self._subscriptions_url)
+        return self._subscriptions_url.value
+
+    @property
+    def suspended_at(self):
+        """
+        :type: datetime.datetime
+        """
+        self._completeIfNotSet(self._suspended_at)
+        return self._suspended_at.value
+
+    @property
+    def team_count(self):
+        """
+        :type: integer
+        """
+        self._completeIfNotSet(self._team_count)
+        return self._team_count.value
 
     @property
     def total_private_repos(self):
@@ -303,7 +393,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: integer
         """
         self._completeIfNotSet(self._total_private_repos)
-        return self._NoneIfNotSet(self._total_private_repos)
+        return self._total_private_repos.value
 
     @property
     def type(self):
@@ -311,7 +401,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._type)
-        return self._NoneIfNotSet(self._type)
+        return self._type.value
 
     @property
     def updated_at(self):
@@ -319,7 +409,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: datetime.datetime
         """
         self._completeIfNotSet(self._updated_at)
-        return self._NoneIfNotSet(self._updated_at)
+        return self._updated_at.value
 
     @property
     def url(self):
@@ -327,32 +417,7 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :type: string
         """
         self._completeIfNotSet(self._url)
-        return self._NoneIfNotSet(self._url)
-
-    # v2: Remove this method
-    def create_gist(self, public, files, description=github.GithubObject.NotSet):
-        """
-        :calls: `POST /users/:user/gists <http://developer.github.com/v3/todo>`_
-        :param public: bool
-        :param files: dict of string to :class:`github.InputFileContent.InputFileContent`
-        :param description: string
-        :rtype: :class:`github.Gist.Gist`
-        """
-        assert isinstance(public, bool), public
-        assert all(isinstance(element, github.InputFileContent) for element in files.itervalues()), files
-        assert description is github.GithubObject.NotSet or isinstance(description, (str, unicode)), description
-        post_parameters = {
-            "public": public,
-            "files": dict((key, value._identity) for key, value in files.iteritems()),
-        }
-        if description is not github.GithubObject.NotSet:
-            post_parameters["description"] = description
-        headers, data = self._requester.requestJsonAndCheck(
-            "POST",
-            self.url + "/gists",
-            input=post_parameters
-        )
-        return github.Gist.Gist(self._requester, headers, data, completed=True)
+        return self._url.value
 
     def get_events(self):
         """
@@ -390,16 +455,21 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
             None
         )
 
-    def get_gists(self):
+    def get_gists(self, since=github.GithubObject.NotSet):
         """
         :calls: `GET /users/:user/gists <http://developer.github.com/v3/gists>`_
+        :param since: datetime.datetime format YYYY-MM-DDTHH:MM:SSZ
         :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Gist.Gist`
         """
+        assert since is github.GithubObject.NotSet or isinstance(since, datetime.datetime), since
+        url_parameters = dict()
+        if since is not github.GithubObject.NotSet:
+            url_parameters["since"] = since.strftime("%Y-%m-%dT%H:%M:%SZ")
         return github.PaginatedList.PaginatedList(
             github.Gist.Gist,
             self._requester,
             self.url + "/gists",
-            None
+            url_parameters
         )
 
     def get_keys(self):
@@ -468,23 +538,32 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         :param name: string
         :rtype: :class:`github.Repository.Repository`
         """
-        assert isinstance(name, (str, unicode)), name
+        assert isinstance(name, (str, six.text_type)), name
         headers, data = self._requester.requestJsonAndCheck(
             "GET",
             "/repos/" + self.login + "/" + name
         )
         return github.Repository.Repository(self._requester, headers, data, completed=True)
 
-    def get_repos(self, type=github.GithubObject.NotSet):
+    def get_repos(self, type=github.GithubObject.NotSet, sort=github.GithubObject.NotSet,
+                  direction=github.GithubObject.NotSet):
         """
         :calls: `GET /users/:user/repos <http://developer.github.com/v3/repos>`_
         :param type: string
+        :param sort: string
+        :param direction: string
         :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Repository.Repository`
         """
-        assert type is github.GithubObject.NotSet or isinstance(type, (str, unicode)), type
+        assert type is github.GithubObject.NotSet or isinstance(type, (str, six.text_type)), type
+        assert sort is github.GithubObject.NotSet or isinstance(sort, (str, six.text_type)), sort
+        assert direction is github.GithubObject.NotSet or isinstance(direction, (str, six.text_type)), direction
         url_parameters = dict()
         if type is not github.GithubObject.NotSet:
             url_parameters["type"] = type
+        if sort is not github.GithubObject.NotSet:
+            url_parameters["sort"] = sort
+        if direction is not github.GithubObject.NotSet:
+            url_parameters["direction"] = direction
         return github.PaginatedList.PaginatedList(
             github.Repository.Repository,
             self._requester,
@@ -545,6 +624,18 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
     def _identity(self):
         return self.login
 
+    def get_organization_membership(self, org):
+        """
+        :calls: `GET /user/memberships/orgs/:org <https://developer.github.com/v3/orgs/members/#get-your-organization-membership>`_
+        :rtype: :class:`github.Membership.Membership`
+        """
+        assert isinstance(org, int)
+        headers, data = self._requester.requestJsonAndCheck(
+            "GET",
+            "/user/memberships/orgs/" + str(org)
+        )
+        return github.Membership.Membership(self._requester, headers, data, completed=True)
+
     def _initAttributes(self):
         self._avatar_url = github.GithubObject.NotSet
         self._bio = github.GithubObject.NotSet
@@ -565,19 +656,27 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
         self._hireable = github.GithubObject.NotSet
         self._html_url = github.GithubObject.NotSet
         self._id = github.GithubObject.NotSet
+        self._invitation_teams_url = github.GithubObject.NotSet
+        self._inviter = github.GithubObject.NotSet
         self._location = github.GithubObject.NotSet
         self._login = github.GithubObject.NotSet
         self._name = github.GithubObject.NotSet
+        self._node_id = github.GithubObject.NotSet
         self._organizations_url = github.GithubObject.NotSet
         self._owned_private_repos = github.GithubObject.NotSet
+        self._permissions = github.GithubObject.NotSet
         self._plan = github.GithubObject.NotSet
         self._private_gists = github.GithubObject.NotSet
         self._public_gists = github.GithubObject.NotSet
         self._public_repos = github.GithubObject.NotSet
         self._received_events_url = github.GithubObject.NotSet
         self._repos_url = github.GithubObject.NotSet
+        self._role = github.GithubObject.NotSet
+        self._site_admin = github.GithubObject.NotSet
         self._starred_url = github.GithubObject.NotSet
         self._subscriptions_url = github.GithubObject.NotSet
+        self._suspended_at = github.GithubObject.NotSet
+        self._team_count = github.GithubObject.NotSet
         self._total_private_repos = github.GithubObject.NotSet
         self._type = github.GithubObject.NotSet
         self._updated_at = github.GithubObject.NotSet
@@ -585,110 +684,90 @@ class NamedUser(github.GithubObject.CompletableGithubObject):
 
     def _useAttributes(self, attributes):
         if "avatar_url" in attributes:  # pragma no branch
-            assert attributes["avatar_url"] is None or isinstance(attributes["avatar_url"], (str, unicode)), attributes["avatar_url"]
-            self._avatar_url = attributes["avatar_url"]
+            self._avatar_url = self._makeStringAttribute(attributes["avatar_url"])
         if "bio" in attributes:  # pragma no branch
-            assert attributes["bio"] is None or isinstance(attributes["bio"], (str, unicode)), attributes["bio"]
-            self._bio = attributes["bio"]
+            self._bio = self._makeStringAttribute(attributes["bio"])
         if "blog" in attributes:  # pragma no branch
-            assert attributes["blog"] is None or isinstance(attributes["blog"], (str, unicode)), attributes["blog"]
-            self._blog = attributes["blog"]
+            self._blog = self._makeStringAttribute(attributes["blog"])
         if "collaborators" in attributes:  # pragma no branch
-            assert attributes["collaborators"] is None or isinstance(attributes["collaborators"], (int, long)), attributes["collaborators"]
-            self._collaborators = attributes["collaborators"]
+            self._collaborators = self._makeIntAttribute(attributes["collaborators"])
         if "company" in attributes:  # pragma no branch
-            assert attributes["company"] is None or isinstance(attributes["company"], (str, unicode)), attributes["company"]
-            self._company = attributes["company"]
+            self._company = self._makeStringAttribute(attributes["company"])
         if "contributions" in attributes:  # pragma no branch
-            assert attributes["contributions"] is None or isinstance(attributes["contributions"], (int, long)), attributes["contributions"]
-            self._contributions = attributes["contributions"]
+            self._contributions = self._makeIntAttribute(attributes["contributions"])
         if "created_at" in attributes:  # pragma no branch
-            assert attributes["created_at"] is None or isinstance(attributes["created_at"], (str, unicode)), attributes["created_at"]
-            self._created_at = self._parseDatetime(attributes["created_at"])
+            self._created_at = self._makeDatetimeAttribute(attributes["created_at"])
         if "disk_usage" in attributes:  # pragma no branch
-            assert attributes["disk_usage"] is None or isinstance(attributes["disk_usage"], (int, long)), attributes["disk_usage"]
-            self._disk_usage = attributes["disk_usage"]
+            self._disk_usage = self._makeIntAttribute(attributes["disk_usage"])
         if "email" in attributes:  # pragma no branch
-            assert attributes["email"] is None or isinstance(attributes["email"], (str, unicode)), attributes["email"]
-            self._email = attributes["email"]
+            self._email = self._makeStringAttribute(attributes["email"])
         if "events_url" in attributes:  # pragma no branch
-            assert attributes["events_url"] is None or isinstance(attributes["events_url"], (str, unicode)), attributes["events_url"]
-            self._events_url = attributes["events_url"]
+            self._events_url = self._makeStringAttribute(attributes["events_url"])
         if "followers" in attributes:  # pragma no branch
-            assert attributes["followers"] is None or isinstance(attributes["followers"], (int, long)), attributes["followers"]
-            self._followers = attributes["followers"]
+            self._followers = self._makeIntAttribute(attributes["followers"])
         if "followers_url" in attributes:  # pragma no branch
-            assert attributes["followers_url"] is None or isinstance(attributes["followers_url"], (str, unicode)), attributes["followers_url"]
-            self._followers_url = attributes["followers_url"]
+            self._followers_url = self._makeStringAttribute(attributes["followers_url"])
         if "following" in attributes:  # pragma no branch
-            assert attributes["following"] is None or isinstance(attributes["following"], (int, long)), attributes["following"]
-            self._following = attributes["following"]
+            self._following = self._makeIntAttribute(attributes["following"])
         if "following_url" in attributes:  # pragma no branch
-            assert attributes["following_url"] is None or isinstance(attributes["following_url"], (str, unicode)), attributes["following_url"]
-            self._following_url = attributes["following_url"]
+            self._following_url = self._makeStringAttribute(attributes["following_url"])
         if "gists_url" in attributes:  # pragma no branch
-            assert attributes["gists_url"] is None or isinstance(attributes["gists_url"], (str, unicode)), attributes["gists_url"]
-            self._gists_url = attributes["gists_url"]
+            self._gists_url = self._makeStringAttribute(attributes["gists_url"])
         if "gravatar_id" in attributes:  # pragma no branch
-            assert attributes["gravatar_id"] is None or isinstance(attributes["gravatar_id"], (str, unicode)), attributes["gravatar_id"]
-            self._gravatar_id = attributes["gravatar_id"]
+            self._gravatar_id = self._makeStringAttribute(attributes["gravatar_id"])
         if "hireable" in attributes:  # pragma no branch
-            assert attributes["hireable"] is None or isinstance(attributes["hireable"], bool), attributes["hireable"]
-            self._hireable = attributes["hireable"]
+            self._hireable = self._makeBoolAttribute(attributes["hireable"])
         if "html_url" in attributes:  # pragma no branch
-            assert attributes["html_url"] is None or isinstance(attributes["html_url"], (str, unicode)), attributes["html_url"]
-            self._html_url = attributes["html_url"]
+            self._html_url = self._makeStringAttribute(attributes["html_url"])
         if "id" in attributes:  # pragma no branch
-            assert attributes["id"] is None or isinstance(attributes["id"], (int, long)), attributes["id"]
-            self._id = attributes["id"]
+            self._id = self._makeIntAttribute(attributes["id"])
+        if "invitation_teams_url" in attributes:  # pragma no branch
+            self._invitation_teams_url = self._makeStringAttribute(attributes["invitation_teams_url"])
+        if "inviter" in attributes:  # pragma no branch
+            self._inviter = self._makeClassAttribute(github.NamedUser.NamedUser, attributes["inviter"])
         if "location" in attributes:  # pragma no branch
-            assert attributes["location"] is None or isinstance(attributes["location"], (str, unicode)), attributes["location"]
-            self._location = attributes["location"]
+            self._location = self._makeStringAttribute(attributes["location"])
         if "login" in attributes:  # pragma no branch
-            assert attributes["login"] is None or isinstance(attributes["login"], (str, unicode)), attributes["login"]
-            self._login = attributes["login"]
+            self._login = self._makeStringAttribute(attributes["login"])
         if "name" in attributes:  # pragma no branch
-            assert attributes["name"] is None or isinstance(attributes["name"], (str, unicode)), attributes["name"]
-            self._name = attributes["name"]
+            self._name = self._makeStringAttribute(attributes["name"])
+        if "node_id" in attributes:  # pragma no branch
+            self._node_id = self._makeStringAttribute(attributes["node_id"])
         if "organizations_url" in attributes:  # pragma no branch
-            assert attributes["organizations_url"] is None or isinstance(attributes["organizations_url"], (str, unicode)), attributes["organizations_url"]
-            self._organizations_url = attributes["organizations_url"]
+            self._organizations_url = self._makeStringAttribute(attributes["organizations_url"])
         if "owned_private_repos" in attributes:  # pragma no branch
-            assert attributes["owned_private_repos"] is None or isinstance(attributes["owned_private_repos"], (int, long)), attributes["owned_private_repos"]
-            self._owned_private_repos = attributes["owned_private_repos"]
+            self._owned_private_repos = self._makeIntAttribute(attributes["owned_private_repos"])
+        if "permissions" in attributes:  # pragma no branch
+            self._permissions = self._makeClassAttribute(github.Permissions.Permissions, attributes["permissions"])
         if "plan" in attributes:  # pragma no branch
-            assert attributes["plan"] is None or isinstance(attributes["plan"], dict), attributes["plan"]
-            self._plan = None if attributes["plan"] is None else github.Plan.Plan(self._requester, self._headers, attributes["plan"], completed=False)
+            self._plan = self._makeClassAttribute(github.Plan.Plan, attributes["plan"])
         if "private_gists" in attributes:  # pragma no branch
-            assert attributes["private_gists"] is None or isinstance(attributes["private_gists"], (int, long)), attributes["private_gists"]
-            self._private_gists = attributes["private_gists"]
+            self._private_gists = self._makeIntAttribute(attributes["private_gists"])
         if "public_gists" in attributes:  # pragma no branch
-            assert attributes["public_gists"] is None or isinstance(attributes["public_gists"], (int, long)), attributes["public_gists"]
-            self._public_gists = attributes["public_gists"]
+            self._public_gists = self._makeIntAttribute(attributes["public_gists"])
         if "public_repos" in attributes:  # pragma no branch
-            assert attributes["public_repos"] is None or isinstance(attributes["public_repos"], (int, long)), attributes["public_repos"]
-            self._public_repos = attributes["public_repos"]
+            self._public_repos = self._makeIntAttribute(attributes["public_repos"])
         if "received_events_url" in attributes:  # pragma no branch
-            assert attributes["received_events_url"] is None or isinstance(attributes["received_events_url"], (str, unicode)), attributes["received_events_url"]
-            self._received_events_url = attributes["received_events_url"]
+            self._received_events_url = self._makeStringAttribute(attributes["received_events_url"])
         if "repos_url" in attributes:  # pragma no branch
-            assert attributes["repos_url"] is None or isinstance(attributes["repos_url"], (str, unicode)), attributes["repos_url"]
-            self._repos_url = attributes["repos_url"]
+            self._repos_url = self._makeStringAttribute(attributes["repos_url"])
+        if "role" in attributes:  # pragma no branch
+            self._role = self._makeStringAttribute(attributes["role"])
+        if "site_admin" in attributes:  # pragma no branch
+            self._site_admin = self._makeBoolAttribute(attributes["site_admin"])
         if "starred_url" in attributes:  # pragma no branch
-            assert attributes["starred_url"] is None or isinstance(attributes["starred_url"], (str, unicode)), attributes["starred_url"]
-            self._starred_url = attributes["starred_url"]
+            self._starred_url = self._makeStringAttribute(attributes["starred_url"])
         if "subscriptions_url" in attributes:  # pragma no branch
-            assert attributes["subscriptions_url"] is None or isinstance(attributes["subscriptions_url"], (str, unicode)), attributes["subscriptions_url"]
-            self._subscriptions_url = attributes["subscriptions_url"]
+            self._subscriptions_url = self._makeStringAttribute(attributes["subscriptions_url"])
+        if "suspended_at" in attributes:  # pragma no branch
+            self._suspended_at = self._makeDatetimeAttribute(attributes["suspended_at"])
+        if "team_count" in attributes:
+            self._team_count = self._makeIntAttribute(attributes["team_count"])
         if "total_private_repos" in attributes:  # pragma no branch
-            assert attributes["total_private_repos"] is None or isinstance(attributes["total_private_repos"], (int, long)), attributes["total_private_repos"]
-            self._total_private_repos = attributes["total_private_repos"]
+            self._total_private_repos = self._makeIntAttribute(attributes["total_private_repos"])
         if "type" in attributes:  # pragma no branch
-            assert attributes["type"] is None or isinstance(attributes["type"], (str, unicode)), attributes["type"]
-            self._type = attributes["type"]
+            self._type = self._makeStringAttribute(attributes["type"])
         if "updated_at" in attributes:  # pragma no branch
-            assert attributes["updated_at"] is None or isinstance(attributes["updated_at"], (str, unicode)), attributes["updated_at"]
-            self._updated_at = self._parseDatetime(attributes["updated_at"])
+            self._updated_at = self._makeDatetimeAttribute(attributes["updated_at"])
         if "url" in attributes:  # pragma no branch
-            assert attributes["url"] is None or isinstance(attributes["url"], (str, unicode)), attributes["url"]
-            self._url = attributes["url"]
+            self._url = self._makeStringAttribute(attributes["url"])
